#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
reload(sys)
sys.setdefaultencoding('utf8')

import os
import dbus
import dbus.service
import traceback
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from service.dbus_service import DbusService
from service.kylin_update_service import KylinUpdateService
from utils.enums import LaunchMode, TaskMode, LWMode, AUPSource, AppType, AppMsg, Signals, LOADINGWIDTH, LOADINGHEIGHT, MAINWIDTH, MAINHEIGHT, DETAILWIDTH, DETAILHEIGHT, SINTERFACE, SINTERFACES, RUNLIST
from utils.common_utils import CommonUtils
from utils.util_dbus import UtilDbus
from utils.log_manager import LogManager
from utils import log_manager
from mainui.updatelistitemwidget import UpdateListItemWidget
from mainui.messagewidget import MessageWidget
from mainui.loadingwidget import LoadingWidget
from mainui.topwidget import TopWidget
from mainui.bottomwidget import BottomWidget
from mainui.detailwidget import DetailWidget
from mainui.system_tray import SystemTray
from mainui.confwidget import ConfigWidget

import urllib
import urllib2
import socket
import time
import uuid

import locale
import gettext
locale.setlocale(locale.LC_ALL, "")
gettext.bindtextdomain("kylin-update-manager", "/usr/share/locale")
gettext.textdomain("kylin-update-manager")
_ = gettext.gettext

PINGBACK_URL = "http://archive.kylinos-intranet.cn/kylin-update-manager-server/main.php"
PKGLIST = []

class MainWindow(QWidget):

    service_key = ""
    hash_pwd = ""
    item_map_ordinary = {}
    item_map_important = {}

    size_ordinary = 0
    size_important = 0
    app_num_ordinary = 0
    app_num_important = 0
    selected_num_ordinary = 0
    selected_num_important = 0

    available_source = AUPSource.NOSOURCE
    is_log_init = False
    is_working = False
    is_working_c = False
    is_error = False
    is_conffile = False
    launch_mode = None
    task_mode = None

    iface = None
    kus = None
    cu = None

    App_Version_list=[]
    App_install_num=0

    previous_process_type = ""

    def __init__(self, launch_mode, parent=None):
        QWidget.__init__(self, parent)

        # 设置全局异常处理函数
        sys.excepthook = self.global_except_hook

        self.launch_mode = launch_mode

        self.init_log()
        self.init_ui()
        self.check_singleton()
        self.cu = CommonUtils()
        self.check_service_key()
        self.iface = DbusService()
        self.kus = KylinUpdateService(self)
        self.init_main_animation()
        self.init_connect()

        self.pre_work()

    #
    # 函数：全局异常捕获
    # etype：异常类型
    # value：异常值
    # trace：堆栈信息
    #
    def global_except_hook(self, etype, value, trace):
        msg = traceback.format_exception(etype, value, trace)
        if self.is_log_init == True:
            LogManager.CRI("".join(msg))

        sys.__excepthook__(etype, value, trace)
        #self.exit_app("程序异常退出！详情见 $HOME/.cache/kum.log")
        self.exit_app(_("Crashed! For details in $HOME/.cache/kum.log"))

    #
    # 函数：初始化UI
    #
    def init_ui(self):
        self.setWindowIcon(QIcon("./res/kylin-update-manager.png"))
        self.setWindowTitle(_("Kylin Update Manager"))
        self.setWindowOpacity(1)
        self.setAutoFillBackground(True)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Widget)

        self.top_widget = TopWidget(self)
        self.bottom_widget = BottomWidget(self)
        self.detail_widget = DetailWidget(self)
        self.config_widget = ConfigWidget(self)
        self.detail_widget.move(-336, 0)
        self.loading_widget = LoadingWidget(self)

        self.mouse_press = False
        #self.animation_msg_box = MessageWidget("发现需要更新的软件。", False)
        self.animation_msg_box = MessageWidget(_("Updated software is available for this computer."), False)

        self.resize(0, 0)
        self.show()

        if(self.launch_mode == LaunchMode.MANUAL):
            self.resize(LOADINGWIDTH, LOADINGHEIGHT)
            self.loading_widget.progressBar.hide()
            self.loading_widget.loadinglabel.show()
            self.show_main_window()

    #
    # 函数：初始化动画效果
    #
    def init_main_animation(self):
        topAnimation = QPropertyAnimation(self.top_widget, "pos")
        topAnimation.setDuration(1000)
        topAnimation.setStartValue(QPoint(0, -141))
        topAnimation.setEndValue(QPoint(0, 0))

        bottomAnimation = QPropertyAnimation(self.bottom_widget, "pos")
        bottomAnimation.setDuration(1000)
        bottomAnimation.setStartValue(QPoint(0, 600))
        bottomAnimation.setEndValue(QPoint(0, 141))

        loadingAnimation = QPropertyAnimation(self.loading_widget, "pos")
        loadingAnimation.setDuration(1000)
        loadingAnimation.setStartValue(QPoint(0, 0))
        loadingAnimation.setEndValue(QPoint(672, 0))

        self.mainGroup = QParallelAnimationGroup()
        self.mainGroup.addAnimation(topAnimation)
        self.mainGroup.addAnimation(bottomAnimation)
        self.mainGroup.addAnimation(loadingAnimation)

        topOrigAnimation = QPropertyAnimation(self.top_widget, "pos")
        topOrigAnimation.setDuration(1000)
        topOrigAnimation.setStartValue(QPoint(0, 0))
        topOrigAnimation.setEndValue(QPoint(0, -141))

        bottomOrigAnimation = QPropertyAnimation(self.bottom_widget, "pos")
        bottomOrigAnimation.setDuration(1000)
        bottomOrigAnimation.setStartValue(QPoint(0, 141))
        bottomOrigAnimation.setEndValue(QPoint(0, 600))

        detailOrigAnimation = QPropertyAnimation(self.detail_widget, "pos")
        detailOrigAnimation.setDuration(1000)
        detailOrigAnimation.setStartValue(QPoint(-336, 0))
        detailOrigAnimation.setEndValue(QPoint(0, 0))

        self.detailGroup = QParallelAnimationGroup(self)
        self.detailGroup.addAnimation(topOrigAnimation)
        self.detailGroup.addAnimation(bottomOrigAnimation)
        self.detailGroup.addAnimation(detailOrigAnimation)

    #
    # 函数：信号槽绑定
    #
    def init_connect(self):
        self.loading_widget.title_bar.min_button.clicked.connect(self.slot_click_min_button)
        self.loading_widget.title_bar.close_button.clicked.connect(self.slot_click_close_button)
        self.top_widget.btn_min.clicked.connect(self.slot_click_min_button)
        self.top_widget.btn_close.clicked.connect(self.slot_click_close_button)
        self.top_widget.config_button.clicked.connect(self.slot_click_config_button)
        self.top_widget.btn_install.clicked.connect(self.slot_click_install_button)
        self.bottom_widget.lw_ordinary.itemClicked.connect(self.slot_click_update_item)
        self.bottom_widget.lw_important.itemClicked.connect(self.slot_click_update_item)
        self.bottom_widget.click_ordinary_main_cb.connect(self.slot_reset_ordinary_cb)
        self.bottom_widget.click_important_main_cb.connect(self.slot_reset_important_cb)
        self.detail_widget.title_bar.min_button.clicked.connect(self.slot_click_min_button)
        self.detail_widget.title_bar.close_button.clicked.connect(self.slot_click_close_button)

        self.animation_msg_box.quitSignal.connect(self.exit_app)
        self.connect(self.iface, Signals.dbus_process_msg, self.slot_dbus_process_msg)

        # 设置界面按钮改变信号
        self.config_widget.bootautocheckbt.checkedChanged.connect(self.on_bootautocheck_changed)
        self.config_widget.dailycheckbt.checkedChanged.connect(self.on_dailycheck_changed)


    #
    # 函数：初始化日志模块
    #
    def init_log(self):
        # log_manager.init_log()
        self.is_log_init = True

    #
    # 函数：保证单例模式
    #
    def check_singleton(self):
        try:
            bus = dbus.SessionBus()
        except:
            LogManager.CRI(_("Dbus Initialize Failed, Program Quit."))
            sys.exit(0)

        # 已有实例在运行，将其显示到前端
        try:
            proxy_obj = bus.get_object(SINTERFACE, SINTERFACES)
            iface = dbus.Interface(proxy_obj, SINTERFACE)
            iface.bring_to_front()
            # 退出重复运行的当前实例
            sys.exit(0)

        # 没有实例在运行，创建一个dbus
        except dbus.DBusException:
            bus_name = dbus.service.BusName(SINTERFACE, bus)
            self.dbusControler = UtilDbus(self, bus_name)

    #
    # 函数：获取麒麟操作系统服务序列号
    #
    def check_service_key(self):
        if(self.cu.is_os_community() == True):
            self.service_key = "kylin"
            self.hash_pwd = "123123"
        else:
            print(_("Geting Service Serial Number..."))
            self.service_key = self.cu.get_service_key()
            self.hash_pwd = self.cu.get_hash_pwd()

            if self.service_key is None:
                LogManager.ERR(_("Get Service Serial Number Failed, Program Quit!"))
                self.exit_app(_("Get Service Serial Number Failed, Program Quit!"))


    #
    # 函数：收集客户端信息发送给php服务器
    #
    def pingback(self):
        host_ip,version,soft_version,host_mac = '','','',''
        #获取本机ip
        try:
            s = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
            s.connect(('8.8.8.8',80))
            host_ip = s.getsockname()[0]
        except:
            host_ip = 'Ip failed to get'
        #获取系统版本
        with open('/etc/lsb-release','r') as f:
            for line in f:
                if line.strip().startswith('DISTRIB_KYLIN_RELEASE='):
                    version = line.strip().split('=')
                    version = version[1]
        #获取软件版本
        output = os.popen('dpkg -l|grep kylin-update-m').readlines()
        if output:
            soft_version = output[0].strip().split()
            soft_version = soft_version[2]

        #获取时间
        nowtime = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))

        #获取Mac
        mac = uuid.UUID(int = uuid.getnode()).hex[-12:]
        host_mac = ":".join([mac[e:e+2] for e in range(0,11,2)])


        send_data = {'ip':host_ip, 'version':version, 'soft_version':soft_version, 'datetime':nowtime, 'host_mac':host_mac}
        send_data_urlencode = urllib.urlencode(send_data)
        requrl = PINGBACK_URL

        try:
            req=urllib2.Request(url = requrl,data = send_data_urlencode)
            res_ddd = urllib2.urlopen(req)
        except Exception, e:
            print(_("Pingback failed."), e)

    #
    # 函数：准备工作
    #
    def pre_work(self):
        # 修复可能出现的dpkg问题，卸载可能存在的SP升级包，完成后由slot_dbus_process_msg() 执行 main_work()
        print(_("executing dpkg configure."))
        self.iface.call_configure_dpkg_by_shell()

    #
    # 函数：主流程
    #
    def main_work(self):
        self.pingback()
        if(self.kus.check_available_medium() == True):
            if(self.kus.is_sinfo_down_success == True):
                if(self.kus.is_sinfo_match == True):
                    self.check_status()
                else:
                    qb = QMessageBox(self)
                    qb.setIcon(QMessageBox.Question)
                    #qb.setWindowTitle("没有匹配的源")
                    qb.setWindowTitle(_("No Matching Source"))
                    #qb.setText("当前连接的服务器或光盘中没有完全匹配的软件源。\n是否选择其中的默认软件源进行升级？\n\n这可能会导致未知的后果，请向技服人员确认。")
                    qb.setText(_("The connected server or CD does not have matched software source.\nDo you want to select the default software source to upgrade?\n\nThe operation may cause unknow result, please confirm to technocrat."))
                    qb.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
                    qb.setButtonText(QMessageBox.Yes, QString(_("OK")))
                    qb.setButtonText(QMessageBox.No, QString(_("Cancel")))
                    rtn = qb.exec_()

                    if(rtn == QMessageBox.Yes):
                        self.check_status()
                    else:
                        self.set_default_sourceslist()
                        self.exit_app(_("User Cancel, Program Quit."))

            # 下载sinfo失败
            else:
                self.set_default_sourceslist()

                if(self.kus.is_sinfo_match == False):
                    if(self.launch_mode == LaunchMode.AUTOMATIC):
                        #self.exit_app("媒介中没有可用的软件源，程序退出.")
                        self.exit_app(_("No software source available, Program Quit."))
                    else:
                        #self.animation_msg_box.set_animation_args("媒介中没有可用的软件源，程序退出", True)
                        self.animation_msg_box.set_animation_args(_("No software source available, Program Quit"), True)
                        self.animation_msg_box.showAnimation(5000)
                else:
                    if(self.launch_mode == LaunchMode.AUTOMATIC):
                        #self.exit_app("下载软件源信息出错，程序退出.")
                        self.exit_app(_("Error when downloading info of software source, Program Quit."))
                    else:
                        #self.animation_msg_box.set_animation_args("下载软件源信息出错，程序退出.", True)
                        self.animation_msg_box.set_animation_args(_("Error when downloading info of software source, Program Quit."), True)
                        self.animation_msg_box.showAnimation(5000)

        # 找不到可用的升级媒介
        else:
            self.set_default_sourceslist()

            if(self.launch_mode == LaunchMode.AUTOMATIC):
                #self.exit_app("没有可用的升级媒介，程序退出.")
                self.exit_app(_("No upgrade media available, Program Quit."))
            else:
                #self.animation_msg_box.set_animation_args("没有可用的升级媒介，程序退出.", True)
                self.animation_msg_box.set_animation_args(_("No upgrade media available, Program Quit."), True)
                self.animation_msg_box.showAnimation(5000)

    #
    # 函数：检测apt update、自我升级、系统升级
    #
    def check_status(self):
        # 修改sourceslist
        self.available_source = self.kus.available_source
        lines = self.kus.get_sources()
        self.iface.change_source_list_to_kylin_update(lines)

        # 需要apt-get update
        if(self.kus.check_need_apt_get_update() == True):
            # 执行apt-get update，False代表完成后由slot_dbus_process_msg() 执行 check_update_type()
            self.iface.call_update_source_by_shell(False)
        else:
            self.kus.check_update_type()

    #
    # 函数：任务分支
    #
    def switch_task(self, is_kylin_update, is_kum_update):
        # 可能的apt-get update已完成，告知dbus后台初始化apt库
        self.iface.call_init_cache()

        # pyqt缺陷，密集执行函数时初始化trayicon很容易导致trayicon大小异常
        # 不得已将trayicon的初始化放此处，在大部分任务处理完之后
        self.tray_icon = SystemTray(self)

        if(is_kum_update == True):
            self.task_mode = TaskMode.UPDATEKUM
        else:
            if(is_kylin_update == True):
                self.task_mode = TaskMode.UPDATEOS
            else:
                self.task_mode = TaskMode.UPDATEPKG

        # 更新管理器自我升级流程
        if(self.task_mode == TaskMode.UPDATEKUM):
            #self.tray_icon.start_tray_msg("正在升级更新管理器.")
            self.tray_icon.start_tray_msg(_("Upgrading Update Manager."))
            self.is_working_c = True
            self.iface.call_upone("kylin-update-manager")

        # 系统升级流程
        elif(self.task_mode == TaskMode.UPDATEOS):
            # 提示进行系统升级
            self.show_main_window()
            (os_version_local, os_version_source) = self.kus.get_os_versions()
            qb = QMessageBox(self)
            qb.setIcon(QMessageBox.Question)
            #qb.setWindowTitle("发现SP补丁包")
            qb.setWindowTitle(_("Service Pack Available"))
            #qb.setText("当前SP版本 :  " + os_version_local + "\n可更新为 :  " + os_version_source + "\n\n是否现在执行更新？   ")
            qb.setText(_("Current SP Version: ") + os_version_local + _("\nCan Upgrade: ") + os_version_source + _("\n\nDo you upgrade now?  "))
            qb.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            qb.setButtonText(QMessageBox.Yes, QString(_("Upgrade")))
            qb.setButtonText(QMessageBox.No, QString(_("Cancel")))
            rtn = qb.exec_()

            if(rtn == QMessageBox.Yes):
                # 下载安装kylin-update包，进入系统升级流程
                self.resize(LOADINGWIDTH, LOADINGHEIGHT)
                self.loading_widget.progressBar.hide()
                self.loading_widget.loadinglabel.show()
                self.show_main_window()
                self.iface.call_insone("kylin-update-" + self.kus.desktop_or_server)

            # 用户不升级系统，进入包升级流程
            else:
                self.kus.start_access()

        # 包升级流程
        else:
            self.kus.start_access()

    #
    # 函数：自我升级完成，退出dbus，重启更新管理器
    #
    def kylin_update_manager_upgrade_finish(self):
        if self.iface:
            self.iface.exit_dbus()

        if self.launch_mode == LaunchMode.AUTOMATIC:
            os.system("kylin-update-manager-restart -auto &")
        else:
            os.system("kylin-update-manager-restart -m &")

        self.close()
        sys.exit("Update Manager Restart.")

    #
    # 函数：修复配置文件冲突，默认选择Yes
    #
    def fix_conffile(self):
        print(_("fixing conffile."))
        self.iface.call_fix_conffile()

    #
    # 函数：安装kylin-update完成，执行并退出更新管理器
    #
    def kylin_update_install_finish(self):
        os.system("/tmp/kylin-update/kylin-simpleupdate &")
        self.exit_app(_("System upgrade start, Update Manager Quit."))

    #
    # 函数：软件安装完成后，需要拉起的软件运行其启动命令
    #
    def run_software(self):
        for line in RUNLIST:
            run_command = line + " &"
            os.system(run_command)
        self.exit_app(_("Update succeed..."))


    #
    # 函数：将sources.list.kumbak还原为sources.list
    #
    def set_default_sourceslist(self):
        print "# restore /etc/apt/sources.list and do apt-get update"
        self.iface.change_source_list_to_default()
        #self.iface.call_update_source_by_shell(True)#lixiang

    '''
    Dbus Apt后台信号统一处理函数
    信号传递顺序：
    daemon.kum_fetch_signal() / kum_apt_signal()
    dbus_service.slot_kum_fetch_signal() / slot_kum_apt_signal()
    self.slot_dbus_process_msg()
    '''
    def slot_dbus_process_msg(self, signal_type, app_name, action, percent, process_type, status):
        print app_name, " : ", signal_type, " : ", process_type, " : ", str(percent), " : ", status, " : ", action

        # conffile冲突
        if process_type == "apt_conffile":
            self.is_conffile = True

        # 错误信号,有conffile时暂不处理
        if process_type == "apt_error" and self.is_conffile == False:
            self.set_default_sourceslist()
            self.is_working = False
            self.is_error = True
            if app_name == "#update" or app_name == "#update-shell":
                #self.animation_msg_box.set_animation_args("源更新失败，程序将在5秒后退出.", True)
                LogManager.INFO(_("Update source failed, five seconds later, Program will quit."))
                self.animation_msg_box.set_animation_args(_("Update source failed, five seconds later, Program will quit."), True)
                self.animation_msg_box.showAnimation(5000)
            else:
                # 有包无法下载
                if(str(percent) == "-99.0"):
                    msgToShow = status + " " + _("Update software failed, five seconds later, Program will quit.")
                    LogManager.INFO(msgToShow)
                    self.animation_msg_box.set_animation_args(msgToShow, True)
                    self.animation_msg_box.showAnimation(5000)
                else:
                    # 升级流程时出错,自动退出.
                    if(self.task_mode == TaskMode.UPDATEOS):
                        #self.animation_msg_box.set_animation_args("SP补丁更新错误，程序将在5秒后退出.", True)
                        LogManager.INFO(_("Update service pack failed, five seconds later, Program will quit."))
                        self.animation_msg_box.set_animation_args(_("Update service pack failed, five seconds later, Program will quit."), True)
                        self.animation_msg_box.showAnimation(5000)
                    else:
                        #self.animation_msg_box.set_animation_args("软件更新失败，程序将在5秒后退出.", True)
                        LogManager.INFO(_("Update software failed, five seconds later, Program will quit."))
                        self.animation_msg_box.set_animation_args(_("Update software failed, five seconds later, Program will quit."), True)
                        self.animation_msg_box.showAnimation(5000)

        # 调用本地命令dpkg --configure -a过程的信号
        if app_name == "#configure-shell":
            if process_type == "finish" and str(percent) == "200.0":
                # dpkg --configure -a完成，开始执行main work
                #self.loading_widget.tiplabel.setText("正在从源中更新数据...")
                self.loading_widget.tiplabel.setText(_("Updating data from source..."))
                self.main_work()

        # 调用本地命令apt-get update过程的信号
        if app_name == "#update-shell":
            if process_type == "finish" and str(percent) == "200.0":
                print "!!  一次apt-get update完成，模式：", action
                # 0既传入的False，代表非quiet，开始检测是否有系统版本升级
                if action == "0":
                    self.kus.check_update_type()

        # 更新管理器自我升级的信号，kylin-update-manager包安装完成，重启自身和dbus后台
        if app_name == "kylin-update-manager":
            if process_type == "apt_finish" and str(percent) == "200.0":
                for i in self.App_Version_list:
                    if i["name"]==app_name:
                        LogManager.INSTALL(("installed " + app_name+' '+i['version']))
            
                self.kylin_update_manager_upgrade_finish()

        # 系统升级流程的信号，kylin-update包安装完成，执行系统升级
        if app_name == "kylin-update-desktop" or app_name == "kylin-update-server":
            if process_type == "apt_finish" and str(percent) == "200.0":
                for i in self.App_Version_list:
                    if i["name"]==app_name:
                        LogManager.INSTALL(("installed " + app_name+' '+i['version']))
    
                self.kylin_update_install_finish()

        # 调用本地命令fix conffile过程的信号
        if app_name == "#fix-conffile-shell":
            if self.is_working_c == True:
                LogManager.INFO(_("Crucial update successed..."))
                self.is_working_c = False
                self.detail_widget.pb_fetch.setValue(0)
                self.detail_widget.pb_apt.setValue(0)
                self.detail_widget.te_detail.clear()
                self.tray_icon.hide()

                if self.isHidden() == True:
                    self.set_default_sourceslist()
                    self.animation_msg_box.set_animation_args(_("Update succeed, five seconds later, Program will quit."), True)
                    self.animation_msg_box.showAnimation(5000)
            else:
                LogManager.INFO(_("Ordinary and Important update successed..."))
                self.is_working = False
                self.detail_widget.pb_apt.setValue(100)
                self.detail_widget.te_detail.append(_("Update Succesed."))
                self.set_default_sourceslist()
                self.animation_msg_box.set_animation_args(_("Update succeed, five seconds later, Program will quit."), True)
                self.animation_msg_box.showAnimation(5000)

        # 包升级过程的信号
        else:
            # 下载过程
            if app_name == "#install_and_upgrade" and signal_type == "fetch":
                if process_type == "down_start":
                    #self.detail_widget.te_detail.append("开始下载.")
                    self.detail_widget.te_detail.append(_("Start Download."))
                    self.detail_widget.pb_fetch.setValue(0)
                elif process_type == "down_stop":
                    #self.detail_widget.te_detail.append("下载完成，准备安装.")
                    self.detail_widget.te_detail.append(_("Download complete, Ready to install."))
                    self.detail_widget.pb_fetch.setValue(100)
                else:
                    if "down_pulse" in process_type:
                        # apt会重复打印信息，过滤掉
                        if process_type != self.previous_process_type:
                            self.previous_process_type = process_type
                            self.detail_widget.te_detail.append(process_type.split(" ")[1])
                    if percent not in [0.0, "0.0", None]:
                        self.detail_widget.pb_fetch.setValue(int(percent))
                        self.tray_icon.process_now = str(int(percent) / 2)

            # 安装过程
            elif app_name == "#install_and_upgrade" and signal_type == "apt" and process_type == "apt_start":
                self.detail_widget.pb_apt.setValue(0)
                self.detail_widget.te_detail.append(status)
                self.detail_widget.te_detail.append(_("Installing..."))

            # 安装结束
            elif app_name == "#install_and_upgrade" and signal_type == "apt" and process_type == "apt_finish" and self.is_error == False:
                for pkg in PKGLIST:
                    for i in self.App_Version_list:
                        if i["name"]==pkg:
                            LogManager.INSTALL(("installed " + pkg+' '+i['version']))

                if self.is_conffile == True:
                    self.fix_conffile()
                else:
                    if self.is_working_c == True:
                        LogManager.INFO(_("Crucial update successed..."))
                        self.is_working_c = False
                        self.detail_widget.pb_fetch.setValue(0)
                        self.detail_widget.pb_apt.setValue(0)
                        self.detail_widget.te_detail.clear()
                        self.tray_icon.hide()

                        if self.isHidden() == True:
                            self.set_default_sourceslist()
                            if RUNLIST:
                                self.run_software()
                            self.animation_msg_box.set_animation_args(_("Update succeed, five seconds later, Program will quit."), True)
                            self.animation_msg_box.showAnimation(5000)
                    else:
                        LogManager.INFO(_("Ordinary and Important update successed..."))
                        self.is_working = False
                        self.detail_widget.pb_apt.setValue(100)
                        self.detail_widget.te_detail.append(_("Update Succesed."))
                        self.set_default_sourceslist()
                        if RUNLIST:
                            self.run_software()
                        self.animation_msg_box.set_animation_args(_("Update succeed, five seconds later, Program will quit."), True)
                        self.animation_msg_box.showAnimation(5000)

            # 流程pulse
            else:
                if signal_type == "apt" and process_type == "apt_pulse":
                    if (app_name not in PKGLIST) and (app_name not in ("dpkg-exec","fontconfig","dbus")):
                        PKGLIST.append(app_name)
                    self.detail_widget.pb_apt.setValue(int(percent))
                    self.tray_icon.process_now = str(50 + (int(percent) / 2))

    #
    # 函数：所有更新信息获取完成
    #
    def slot_access_completed(self, size_byte_o, size_byte_i, len_o, len_i, crucial_list):
        # 没有任何更新
        if(len_o == 0 and len_i == 0 and len(crucial_list) == 0):
            self.hide()
            self.set_default_sourceslist()
            if self.launch_mode == LaunchMode.MANUAL:
                #self.loading_widget.tiplabel.setText("没有需要更新的软件")
                self.loading_widget.tiplabel.setText(_("No software need updated"))
                #self.animation_msg_box.set_animation_args("没有需要更新的软件，程序将自动退出.", True)
                self.animation_msg_box.set_animation_args(_("No software need updated, program will quit."), True)
                self.animation_msg_box.showAnimation(5000)
            else:
                #self.exit_app("没有可用的更新.")
                self.exit_app(_("No updated available."))
        else:
            self.app_num_ordinary = len_o
            self.app_num_important = len_i
            self.selected_num_ordinary = len_o
            self.selected_num_important = len_i
            self.size_ordinary = size_byte_o
            self.size_important = size_byte_i

            # 优先安装关键更新
            if (len(crucial_list) > 0):
                if (self.launch_mode == LaunchMode.AUTOMATIC):
                    #self.tray_icon.start_tray_msg("正在执行关键更新.")
                    self.tray_icon.start_tray_msg(_("Installing crucial update."))
                self.is_working_c = True
                self.iface.call_install_and_upgrade(crucial_list)

            if len_o > 0 and len_i > 0:
                self.change_display_content(LWMode.BOTH)
            elif len_o == 0 and len_i > 0:
                self.change_display_content(LWMode.IMPORTANT)
            elif len_o > 0 and len_i == 0:
                self.change_display_content(LWMode.ORDINARY)
            elif len_o == 0 and len_i == 0:
                # hide表示只有关键更新
                self.hide()

            print(_("Ordinary update : "), len_o, _(",Important update : "), len_i, _(",Crucial update : "), len(crucial_list))

    #
    # 函数：每获得一个app信息
    #
    def slot_get_one_app(self, msg):
        app_msg = AppMsg()
        app_msg.ZH_CN_NAME = msg["displayName"]
        app_msg.NAME = msg["name"]
        app_msg.ICON = msg["icon"]
        app_msg.SIZE = msg["downloadSizeHuman"]
        app_msg.BYTESIZE = msg["downloadSize"]
        app_msg.INSTALLEDVERSION = msg["installedVersion"]
        app_msg.CANDIDATEVERSION = msg["candidateVersion"]
        app_msg.DESCRIPTION = msg["description"]
        App_Version_dic={'name':'','version':''}
        App_Version_dic["name"]=app_msg.NAME
        App_Version_dic["version"]=app_msg.CANDIDATEVERSION
        self.App_Version_list.insert(self.App_install_num,App_Version_dic)
        self.App_install_num+=1
        if msg['apptype'] == AppType.ORDINARY:
            self.add_ordinary_item(app_msg)
        elif msg['apptype'] == AppType.IMPORTANT:
            self.add_important_item(app_msg)

    #
    # 函数：将一个普通app添加到ui
    #
    def add_ordinary_item(self, app_msg):
        item = QListWidgetItem()
        list_item = UpdateListItemWidget(app_msg, self)
        list_item.setStatusTip(str(app_msg.BYTESIZE))
        item.setWhatsThis(app_msg.DESCRIPTION)
        list_item.size_change.connect(self.recalculate_ordinary_apps_size)
        list_item.checkBox.clicked.connect(self.set_checkbox_ordinary)
        self.bottom_widget.lw_ordinary.addItem(item)
        self.bottom_widget.lw_ordinary.setItemWidget(item, list_item)
        self.item_map_ordinary[app_msg.NAME] = list_item

    #
    # 函数：将一个重要app添加到ui
    #
    def add_important_item(self, app_msg):
        item = QListWidgetItem()
        list_item = UpdateListItemWidget(app_msg, self)
        list_item.setStatusTip(str(app_msg.BYTESIZE))
        item.setWhatsThis(app_msg.DESCRIPTION)
        list_item.size_change.connect(self.recalculate_important_apps_size)
        list_item.checkBox.clicked.connect(self.set_checkbox_important)
        self.bottom_widget.lw_important.addItem(item)
        self.bottom_widget.lw_important.setItemWidget(item, list_item)
        self.item_map_important[app_msg.NAME] = list_item

    #
    # 函数：ui内容初始化
    #
    def change_display_content(self, lw_mode):
        self.top_widget.label_all_num.setText(str(self.app_num_ordinary + self.app_num_important))
        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)

        if lw_mode == LWMode.BOTH:
            size_ordinary_human = self.cu.byte_human(self.size_ordinary)
            size_important_human = self.cu.byte_human(self.size_important)
            #self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")
            self.bottom_widget.label_ordinary.setText(_("Ordinary update details(quantity: ") + str(self.app_num_ordinary) + _(",Selected: ") + str(self.selected_num_ordinary) + _("entries, Size: ") + size_ordinary_human + _(")"))
            #self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")
            self.bottom_widget.label_important.setText(_("Important update details(quantity: ") + str(self.app_num_important) + _(",Selected: ") + str(self.selected_num_important) + _("entries, Size: ") + size_important_human + _(")"))
            self.bottom_widget.lw_ordinary.setCurrentRow(0)
            self.bottom_widget.lw_important.setCurrentRow(0)
            first_item = self.bottom_widget.lw_important.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_both_title()
        elif lw_mode == LWMode.IMPORTANT:
            size_important_human = self.cu.byte_human(self.size_important)
            #self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")
            self.bottom_widget.label_important.setText(_("Important update details(quantity: ") + str(self.app_num_important) + _(",Selected: ") + str(self.selected_num_important) + _("entries, Size: ") + size_important_human + _(")"))
            self.bottom_widget.lw_important.setCurrentRow(0)
            first_item = self.bottom_widget.lw_important.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_important_title()
        else:
            size_ordinary_human = self.cu.byte_human(self.size_ordinary)
            #self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")
            self.bottom_widget.label_ordinary.setText(_("Ordinary update details(quantity: ") + str(self.app_num_ordinary) + _(",Selected: ") + str(self.selected_num_ordinary) + _("entries, Size: ") + size_ordinary_human + _(")"))
            self.bottom_widget.lw_ordinary.setCurrentRow(0)
            first_item = self.bottom_widget.lw_ordinary.item(0)
            self.bottom_widget.te_description.setText(first_item.whatsThis())
            self.bottom_widget.show_ordinary_title()

        self.show_main_window()
        self.switch_to_list_window()

    #
    # 函数：description切换
    #
    def change_description_content(self, status):
        if status:
            firestItem = self.bottom_widget.lw_important.item(self.bottom_widget.lw_important.currentRow())
            self.bottom_widget.te_description.setText(firestItem.whatsThis())
        else:
            firestItem = self.bottom_widget.lw_ordinary.item(self.bottom_widget.lw_ordinary.currentRow())
            self.bottom_widget.te_description.setText(firestItem.whatsThis())

    #
    # 函数：加减计算所选的普通升级包信息
    #
    def recalculate_ordinary_apps_size(self, size_str, flag):
        if flag:
            self.size_ordinary += long(size_str)
            self.selected_num_ordinary += 1
        else:
            self.size_ordinary -= long(size_str)
            self.selected_num_ordinary -= 1

        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)
        size_ordinary_human = self.cu.byte_human(self.size_ordinary)
        #self.bottom_widget.label_ordinary.setText("普通更新详情 (数量：" + str(self.app_num_ordinary) + "，已选中：" + str(self.selected_num_ordinary) + "项，大小：" + size_ordinary_human + ")")
        self.bottom_widget.label_ordinary.setText(_("Ordinary update details(quantity: ") + str(self.app_num_ordinary) + _(",Selected: ") + str(self.selected_num_ordinary) + _("entries, Size: ") + size_ordinary_human + _(")"))

    #
    # 函数：加减计算所选的重要升级包信息
    #
    def recalculate_important_apps_size(self, size_str, flag):
        if flag:
            self.size_important += long(size_str)
            self.selected_num_important += 1
        else:
            self.size_important -= long(size_str)
            self.selected_num_important -= 1

        self.top_widget.label_selected_num.setText(str(self.selected_num_ordinary + self.selected_num_important))
        size_all_human = self.cu.byte_human(self.size_ordinary + self.size_important)
        self.top_widget.label_selected_size.setText(size_all_human)
        size_important_human = self.cu.byte_human(self.size_important)
        #self.bottom_widget.label_important.setText("重要更新详情 (数量：" + str(self.app_num_important) + "，已选中：" + str(self.selected_num_important) + "项，大小：" + size_important_human + ")")
        self.bottom_widget.label_important.setText(_("Important update details(quantity: ") + str(self.app_num_important) + _(",Selected: ") + str(self.selected_num_important) + _("entries, Size: ") + size_important_human + _(")"))

    #
    # 函数：改变普通升级区域checkbox的状态
    #
    def set_checkbox_ordinary(self):
        count = len(self.item_map_ordinary)
        m = 0
        for item in self.item_map_ordinary:
            curItem = self.item_map_ordinary[item]
            if curItem.get_checkbox_status() == True:
                m += 1
        if m == 0:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(0)
        elif m == count:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(2)
        else:
            self.bottom_widget.cb_ordinary.setCheckboxStatus(1)

    #
    # 函数：改变重要升级区域checkbox的状态
    #
    def set_checkbox_important(self):
        count = len(self.item_map_important)
        m = 0
        for item in self.item_map_important:
            curItem = self.item_map_important[item]
            if curItem.get_checkbox_status() == True:
                m += 1
        if m == 0:
            self.bottom_widget.cb_important.setCheckboxStatus(0)
        elif m == count:
            self.bottom_widget.cb_important.setCheckboxStatus(2)
        else:
            self.bottom_widget.cb_important.setCheckboxStatus(1)

    #
    # 函数：显示到前端
    #
    def show_to_frontend(self):
        if self.isMinimized() == True:
            self.showNormal()

        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint | Qt.Widget)
        self.show()
        self.raise_()

    #
    # 函数：重复运行程序
    #
    def single_work(self):
        if self.isHidden() == True:
            print "已经有一个程序在运行了."
            print(_("A daemon is already running."))
            #self.animation_msg_box.set_animation_args("已经有一个程序在运行了.", False)
            self.animation_msg_box.set_animation_args(_("A daemon is already running."), False)
            self.animation_msg_box.showAnimation(5000)
        else:
            self.show_to_frontend()

    #
    # 函数：退出程序
    #
    def exit_app(self, msg=None):
        if self.iface:
            self.iface.exit_dbus()
        self.close()
        # if self.launch_mode == LaunchMode.AUTOMATIC:
            # os.system("sudo init 0")
        sys.exit(msg)

    #
    # 函数：退出程序和dbus后台
    #
    def exit_app_and_dbus(self, msg=None):
        if self.iface:
            self.iface.exit_dbus()
        self.close()

        sys.exit(msg)

    #
    # 函数：切换到list界面
    #
    def switch_to_list_window(self):
        self.resize(MAINWIDTH, MAINHEIGHT)
        self.move_to_center()
        self.mainGroup.start()

    #
    # 函数：切换到详细界面
    #
    def switch_to_detail_window(self):
        self.resize(DETAILWIDTH, DETAILHEIGHT)
        self.move_to_center()
        self.detailGroup.start()

    #
    # 函数：窗口居中
    #
    def move_to_center(self):
        windowWidth = QApplication.desktop().screenGeometry(0).width()
        windowHeight = QApplication.desktop().screenGeometry(0).height()
        self.move((windowWidth - self.width()) / 2, (windowHeight - self.height()) / 2)

    #
    # 函数：显示主界面
    #
    def show_main_window(self):
        self.move_to_center()
        self.show()
        self.raise_()

    #
    # 函数：最小化界面
    #
    def slot_click_min_button(self):
        self.showMinimized()

    #
    # 函数：点击关闭
    #
    def slot_click_close_button(self):
        # 正在执行关键更新，点击关闭则hide主界面
        if self.is_working_c == True:
            self.hide()
        else:
            if self.is_working == True:
                #self.animation_msg_box.set_animation_args("更新未完成，请稍等.", False)
                self.animation_msg_box.set_animation_args(_("Updating, Please wait."), False)
                self.animation_msg_box.showAnimation(5000)
            else:
                self.set_default_sourceslist()
                self.exit_app()

    #
    # 函数：点击设置
    #
    def slot_click_config_button(self):
        self.config_widget.move(self.x()+(self.width()-self.config_widget.width())/2,self.y()+(self.height()-self.config_widget.height())/2)
        self.config_widget.show()

    #
    # 函数：点击安装
    #
    def slot_click_install_button(self):
        work_list = []
        for item in self.item_map_ordinary:
            one_item = self.item_map_ordinary[item]
            if one_item.get_checkbox_status():
                work_list.append(one_item.app_msg.NAME)
        for item in self.item_map_important:
            one_item = self.item_map_important[item]
            if one_item.get_checkbox_status():
                work_list.append(one_item.app_msg.NAME)

        if work_list:
            self.is_working = True
            self.switch_to_detail_window()
            self.iface.call_install_and_upgrade(work_list)
        else:
            #self.animation_msg_box.set_animation_args("至少选择一个更新.", False)
            self.animation_msg_box.set_animation_args(_("Select one entry at least."), False)
            self.animation_msg_box.showAnimation(5000)

    #
    # 函数：点击升级元素
    #
    def slot_click_update_item(self, item):
        description = item.whatsThis()
        self.bottom_widget.te_description.setText(description)

    #
    # 函数：重置cb状态
    #
    def slot_reset_ordinary_cb(self, status):
        if status == 0:
            for item in self.item_map_ordinary:
                curItem = self.item_map_ordinary[item]
                curItem.checkBox.setCheckState(Qt.Unchecked)
        elif status == 2:
            for item in self.item_map_ordinary:
                curItem = self.item_map_ordinary[item]
                curItem.checkBox.setCheckState(Qt.Checked)

    #
    # 函数：重置重要cb状态
    #
    def slot_reset_important_cb(self, status):
        if status == 0:
            for item in self.item_map_important:
                curItem = self.item_map_important[item]
                curItem.checkBox.setCheckState(Qt.Unchecked)
        elif status == 2:
            for item in self.item_map_important:
                curItem = self.item_map_important[item]
                curItem.checkBox.setCheckState(Qt.Checked)

    #
    # 函数：实现无标题栏窗口拖动
    #
    def mousePressEvent(self, event):
        if(event.button() == Qt.LeftButton):
            self.mouse_press = True
            self.drag_pos = event.globalPos() - self.pos()
            event.accept()
    def mouseReleaseEvent(self, event):
        self.mouse_press = False
        self.setWindowOpacity(1)
    def mouseMoveEvent(self, event):
        if(self.mouse_press):
            self.move_pos = event.globalPos()
            self.move(self.move_pos - self.drag_pos)
            self.setWindowOpacity(0.9)
            event.accept()


    def on_bootautocheck_changed(self,checked):
        if checked :
            self.iface.call_autostart_open()
        else:
            self.iface.call_autostart_close()

    def on_dailycheck_changed(self,checked):
        if checked :
            self.iface.call_dailystart_open()
        else:
            self.iface.call_dailystart_close()

def main():
    app = QApplication(sys.argv)
    QTextCodec.setCodecForTr(QTextCodec.codecForName("UTF-8"))
    QTextCodec.setCodecForCStrings(QTextCodec.codecForName("UTF-8"))

    globalfont = QFont()
    globalfont.setPixelSize(14)
    globalfont.setFamily("方正黑体_GBK")
    app.setFont(globalfont)

    app.setOrganizationName("Kylin")
    app.setOrganizationDomain("kylinos.cn")
    app.setApplicationName("kylin-update-manager")

    # 获取启动参数
    if(len(sys.argv) == 1):
        MainWindow(LaunchMode.MANUAL)
        sys.exit(app.exec_())
    elif(len(sys.argv) == 2):
        if(sys.argv[1] == "-m"):
            MainWindow(LaunchMode.MANUAL)
            sys.exit(app.exec_())
        elif(sys.argv[1] == "-auto"):
            MainWindow(LaunchMode.AUTOMATIC)
            sys.exit(app.exec_())
        else:
            #print "参数错误...", sys.argv
            print(_("Parameter Error..."), sys.argv)
    else:
        print(_("Parameter Error..."), sys.argv)


if __name__ == '__main__':
    main()
